%Ignacio Serrano-Pedraza & Vicente Sierra-Vazquez 2010
%%This program generates single-band chimaeras using the RIESZ transform
%%and monogenic signal
%%%i.s.pedraza@ncl.ac.uk

%%01-07-2010

clear all
close all
warning off
%**********************************************************************************************
for i=1:255
    r(i)=i/255;
    g(i)=i/255;
    b(i)=i/255;
end

gray2=[r',g',b'];
colormap(gray2);
map=colormap;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%Both images have to have the same size.

 ImageA=double(imread('IMAGES/O512bn.pcx','pcx'));
 ImageB=double(imread('IMAGES/B512bn.pcx','pcx'));

%ImageA=double(imread('IMAGES/S.pcx','pcx'));
%ImageB=double(imread('IMAGES/B.pcx','pcx'));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Image2D=ImageA;
SIZE=size(Image2D);

degreesimagex=1;%%Using 1 degree is like define the spatial frequency in c/iw (cycles per image width)
degreesimagey=1;
SIZEX=SIZE(1);
SIZEY=SIZE(2);

gx=degreesimagex/SIZEX;%Degrees of visual angle x
gy=degreesimagey/SIZEY;%Degrees of visual angle y

fNu=(SIZEX/degreesimagex)/2; %Nyquist frequency axis u
fNv=(SIZEY/degreesimagey)/2; %Nyquist frequency axis v
fu=fNu*2/SIZEX; %sampling rate in frequency, axis u cycles/degree
fv=fNv*2/SIZEY; %sampling rate in frequency, axis v cycles/degree

[u,v]=meshgrid(-fNu:(2*fNu/SIZEX):fNu-(2*fNu/SIZEX),fNv:-(2*fNv/SIZEY):-(fNv-(2*fNv/SIZEY)));
anglefilt=0*pi/180;
u0=0;
v0=0;
ucord=(u-u0)*cos(anglefilt)+(v-v0)*sin(anglefilt);
vcord=-(u-u0)*sin(anglefilt)+(v-v0)*cos(anglefilt);


%%%%%%%%%%%%%LOGNORMAL FILTER%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
BOCT=6;%%%Bandwidth in octaves
alfa=BOCT./((2*sqrt(2))/sqrt(log(2)));
FC=16;%%Peak frequency
LOGNORMAL=real(exp(-(log((sqrt(ucord.^2+vcord.^2))./FC)).^2./(2*alfa.^2)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


figure(1)
subplot(2,2,1)
image(Image2D)
title('Image A');
axis square
axis off
colormap(gray2)
map=colormap;

%%%%%%%%%%%%%%%%%FOURIER SPECTRUM OF THE IMAGE%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I=Image2D;
Lave=sum(sum(I.*gx*gy))./(degreesimagex*degreesimagey);
c=(I-Lave)./Lave;%Contrast function
FFTimage=(fftshift(fft2(c)));
PHASEimage=angle(FFTimage);
FFTabs=log10(1+abs(FFTimage));

%%%%%%%%%%%%%%%%%%%%%NORMALIZED MODULUS%%%%%%%%%%%%%%%%%%%
I=FFTabs./max(max(abs(FFTabs)));
Modulus=127.5*(1+2*(I-0.5));
figure(1)
subplot(2,2,2)
image(Modulus)
title('Amplitude spectrum');
axis square
axis off
colormap(gray2)
map=colormap;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[R,R1,R2,Envelope,Local_Orientation,Local_Phase,PM_carrier]=Riesz(Image2D,LOGNORMAL);

EnvelopeA=Envelope;
PM_carrierA=PM_carrier;

Envelopenorm=127.5*(1+Envelope./max(max(abs(Envelope))));
figure(1)
subplot(2,2,3)
image(Envelopenorm)
title('Envelope A');
axis square
axis off
colormap(gray2)


PM_carriernorm = 127.5*(1+PM_carrier./max(max(abs(PM_carrier))));
figure(1)
subplot(2,2,4)
image(PM_carriernorm)
title('PM carrier A');
axis square
axis off
colormap(gray2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Image2D=ImageB;

figure(2)
subplot(2,2,1)
image(Image2D)
title('Image B');
axis square
axis off
colormap(gray2)
map=colormap;

%%%%%%%%%%%%%%%%%FOURIER SPECTRUM OF THE IMAGE%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I=Image2D;
Lave=sum(sum(I.*gx*gy))./(degreesimagex*degreesimagey);
c=(I-Lave)./Lave;%Contrast function
FFTimage=(fftshift(fft2(c)));
PHASEimage=angle(FFTimage);
FFTabs=log10(1+abs(FFTimage));

%%%%%%%%%%%%%%%%%%%%%NORMALIZED MODULUS%%%%%%%%%%%%%%%%%%%
I=FFTabs./max(max(abs(FFTabs)));
Modulus=127.5*(1+2*(I-0.5));
figure(2)
subplot(2,2,2)
image(Modulus)
title('Amplitude spectrum');
axis square
axis off
colormap(gray2)
map=colormap;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


[R,R1,R2,Envelope,Local_Orientation,Local_Phase,PM_carrier]=Riesz(Image2D,LOGNORMAL);

EnvelopeB=Envelope;
PM_carrierB=PM_carrier;

Envelopenorm=127.5*(1+Envelope./max(max(abs(Envelope))));
figure(2)
subplot(2,2,3)
image(Envelopenorm)
title('Envelope B');
axis square
axis off
colormap(gray2)


PM_carriernorm = 127.5*(1+PM_carrier./max(max(abs(PM_carrier))));
figure(2)
subplot(2,2,4)
image(PM_carriernorm)
title('PM carrier B');
axis square
axis off
colormap(gray2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%GENERATING SINGLE-BAND VISUAL CHIMAERAS%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% Reconstruction of original band pass images%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Reconstruction=EnvelopeA.*(PM_carrierA);
Reconstruct  = 127.5*(1+Reconstruction./max(max(abs(Reconstruction))));
figure(3)
subplot(2,2,1)
image(Reconstruct)
title('Envelope A x PM carrier A');
axis square
axis off
colormap(gray2)
map=colormap;


Reconstruction=EnvelopeB.*(PM_carrierB);
Reconstruct  = 127.5*(1+Reconstruction./max(max(abs(Reconstruction))));
figure(3)
subplot(2,2,2)
image(Reconstruct)
title('Envelope B x PM carrier B');
axis square
axis off
colormap(gray2)
map=colormap;


%%%%%%%%%%%%%%%%%%%%%%%CHIMAERIC IMAGES%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Reconstruction=EnvelopeA.*(PM_carrierB);
Reconstruct  = 127.5*(1+Reconstruction./max(max(abs(Reconstruction))));
figure(3)
subplot(2,2,3)
image(Reconstruct)
title('Envelope A x PM carrier B');
axis square
axis off
colormap(gray2)
map=colormap;


Reconstruction=EnvelopeB.*(PM_carrierA);
Reconstruct  = 127.5*(1+Reconstruction./max(max(abs(Reconstruction))));
figure(3)
subplot(2,2,4)
image(Reconstruct)
title('Envelope B x PM carrier A');
axis square
axis off
colormap(gray2)
map=colormap;

