%Ignacio Serrano-Pedraza & Vicente Sierra-Vazquez 2010

%%This program obtains the Envelope and the PM carrier 
%%of any image using the RIESZ transform and the monogenic signal.

%%%i.s.pedraza@ncl.ac.uk

%%%01-07-2010

clear all
close all
warning off
%**********************************************************************************************
for i=1:255
    r(i)=i/255;
    g(i)=i/255;
    b(i)=i/255;
end

gray2=[r',g',b'];
colormap(gray2);
map=colormap;

%%Color maps for the Local Orientation and Local Phase
  r([1:64])=1;r([65:128])=[1:-1/63:0];r([129:192])=0;r([193:255])=[0:1/63:1-1/63];
  g([1:64])=[1:-1/63:0];g([65:128])=0;g([129:192])=[0:1/63:1];g([193:255])=1;
  b([1:64])=0;b([65:128])=[0:1/63:1];b([129:192])=[1:-1/63:0];b([193:255])=0;
  colour1=[r',g',b'];
   
  r([1:64])=0;r([65:128])=[0:1/63:1];r([129:192])=1;r([193:255])=[1:-1/63:+1/63];
  g([1:64])=[0:1/63:1];g([65:128])=1;g([129:192])=[1:-1/63:0];g([193:255])=0;
  b([1:64])=[1:-1/63:0];b([65:128])=0;b([129:192])=0;b([193:255])=[0:1/63:1-1/63];
  colour2=[r',g',b'];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


 Image2D=double(imread('IMAGES/O512bn.pcx'));


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

SIZE=size(Image2D);

degreesimagex=1;%%Using 1 degree is like define the spatial frequency in c/iw (cycles per image width)
degreesimagey=1;
SIZEX=SIZE(1);
SIZEY=SIZE(2);

gx=degreesimagex/SIZEX;%Degrees of visual angle x
gy=degreesimagey/SIZEY;%Degrees of visual angle y

fNu=(SIZEX/degreesimagex)/2; %Nyquist frequency axis u
fNv=(SIZEY/degreesimagey)/2; %Nyquist frequency axis v
fu=fNu*2/SIZEX; %sampling rate in frequency, axis u cycles/degree
fv=fNv*2/SIZEY; %sampling rate in frequency, axis v cycles/degree

[u,v]=meshgrid(-fNu:(2*fNu/SIZEX):fNu-(2*fNu/SIZEX),fNv:-(2*fNv/SIZEY):-(fNv-(2*fNv/SIZEY)));
anglefilt=0*pi/180;
u0=0;
v0=0;
ucord=(u-u0)*cos(anglefilt)+(v-v0)*sin(anglefilt);
vcord=-(u-u0)*sin(anglefilt)+(v-v0)*cos(anglefilt);


%%%%%%%%%%%%%LOGNORMAL FILTER%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
BOCT=2.5;%%Bandwidth in octaves
alfa=BOCT./((2*sqrt(2))/sqrt(log(2)));
FC=8;%%Peak frequency  c/deg or c/iw
LOGNORMAL=real(exp(-(log((sqrt(ucord.^2+vcord.^2))./FC)).^2./(2*alfa.^2)));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%



figure(1)
subplot(1,2,1)
image(Image2D)
title('Original Image');
axis square
axis off
colormap(gray2)
map=colormap;

%%%%%%%%%%%%%%%%% 2D FOURIER SPECTRUM OF THE IMAGE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
I=Image2D;
Lave=sum(sum(I.*gx*gy))./(degreesimagex*degreesimagey);
c=(I-Lave)./Lave;%Contrast function
FFTimage=(fftshift(fft2(c)));
PHASEimage=angle(FFTimage);
FFTabs=log10(1+abs(FFTimage));

%%%%%%%%%%%%%%%%%%%%% NORMALIZED MODULUS %%%%%%%%%%%%%%%%%%%
I=FFTabs./max(max(abs(FFTabs)));
Modulus=127.5*(1+2*(I-0.5));
figure(1)
subplot(1,2,2)
image(Modulus)
title('Amplitude spectrum');
axis square
axis off
colormap(gray2)
map=colormap;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[R,R1,R2,Envelope,Local_Orientation,Local_Phase,PM_carrier]=Riesz(Image2D,LOGNORMAL);

Rnorm=127.5*(1+R./max(max(abs(R))));
figure(2)
subplot(2,2,1)
image(Rnorm)
title('Filtered Image (f_B_P)');
axis square
axis off
colormap(gray2)


I=Rnorm;
Lave=sum(sum(I.*gx*gy))./(degreesimagex*degreesimagey);
c=(I-Lave)./Lave;%Contrast function
FFTimage=(fftshift(fft2(c)));
PHASEimage=angle(FFTimage);
FFTabs=log10(1+abs(FFTimage));


I=FFTabs./max(max(abs(FFTabs)));
Modulus=127.5*(1+2*(I-0.5));
figure(2)
subplot(2,2,2)
image(Modulus)
title('Amplitude spectrum');
axis square
axis off
colormap(gray2)
map=colormap;


Envelopenorm=127.5*(1+Envelope./max(max(abs(Envelope))));
figure(2)
subplot(2,2,3)
image(Envelopenorm)
title('Envelope');
axis square
axis off
colormap(gray2)



PM_carriernorm = 127.5*(1+PM_carrier./max(max(abs(PM_carrier))));
figure(2)
subplot(2,2,4)
image(PM_carriernorm)
title('PM carrier');
axis square
axis off
colormap(gray2)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%% fBP and RIESZ TRANSFORMS  fR1 & fR2 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Monogenic signal in pointwise Eulidean coordinates (fBp, fR1, & fR2)%%%%%%%%%%%%%%%%%%

figure(3)
subplot(1,3,1)
image(Rnorm)
title('f_B_P');%%band-pass filtered image
axis square
axis off
colormap(gray2)

R1norm = 127.5*(1+R1./max(max(abs(R1))));
figure(3)
subplot(1,3,2)
image(R1norm)
title('f_R_1');
axis square
axis off
colormap(gray2)

R2norm = 127.5*(1+R2./max(max(abs(R2))));
figure(3)
subplot(1,3,3)
image(R2norm)
title('f_R_2');
axis square
axis off
colormap(gray2)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%NORMALIZED ENVELOPE from 0 to 255%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Envelopenorm=(255*Envelope./max(max(abs(Envelope))));
figure(4)
image(Envelopenorm)
title('Envelope (Normalized 0-255)');
axis square
axis off
colormap(gray2)
hcb = colorbar('Location','SouthOutside','XTickMode','manual');
set(hcb,'xtick',[2 64 128 192 255]);
set(hcb,'XTickLabel', {'0','','0.5','','1'})



%%%%%%%%%%%%%%%%%%%%%%%%%%%%LOCAL ORIENTATION AND LOCAL PHASE %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Local_Orientation_norm = 127.5*(1+Local_Orientation./max(max(abs(Local_Orientation))));
figure(5)
image(Local_Orientation_norm)
title('Local Orientation');
colormap(colour1); 
axis square
axis off
hcb = colorbar('Location','SouthOutside','XTickMode','manual');
set(hcb,'xtick',[2 64 128 192 255]);
set(hcb,'XTickLabel', {'-pi/2','','0','','pi/2'})



Local_Phasenorm  = 127.5*(1+Local_Phase./max(max(abs(Local_Phase))));
figure(6)
image(Local_Phasenorm)
title('Local Phase (wrapped)');
colormap(colour2); 
axis square
axis off
hcb = colorbar('Location','SouthOutside','XTickMode','manual');
set(hcb,'xtick',[2 64 128 192 255]);
set(hcb,'XTickLabel', {'-pi','','0','','pi'})



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%RECONSTRUNCTION OF THE FILTERED IMAGE%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Reconstruction=Envelope.*(PM_carrier);
Reconstruct  = 127.5*(1+Reconstruction./max(max(abs(Reconstruction))));
figure(7)
image(Reconstruct)
title('Reconstruction: Envelope x PM carrier');
axis square
axis off
colormap(gray2)
map=colormap;


